#!/usr/bin/perl
#
###############################################################################
$| = 1;
use strict;
use lib qw(./lib);

use XMLRPC::Lite;
use Jcode;
use CGI;
use CGI::Carp qw(fatalsToBrowser);

use vars qw($URI %Dispatch);

$URI = 'http://www.example.com/path/wiki/wiki_xmlrpc.cgi';

%Dispatch = (
	getrecentchanges => \&getRecentChanges,
	getpage          => \&getPage,
	getpagehtml      => \&getPageHTML,
	getallpages      => \&getAllPages,
	listlinks        => \&listLinks,
	putpage          => \&putPage,
	addpage          => \&addPage,
);

###############################################################################
main();
sub main{
	my $cgi    = new CGI;
	my $j      = new Jcode;
	my $method = $cgi->param('method');

	print $cgi->header( -type  =>'text/html; charset=Shift_JIS', );
	print $cgi->start_html('XML-RPC Client for FSWiki');

	if(defined $Dispatch{lc($method)}){
		$Dispatch{lc($method)}->($cgi,$j);
	}
	else{
		if(!defined $method or $method eq ''){
			print $cgi->h1("Method must be specified.");
		}
		else{
			print $cgi->h1("$method is not defined.");
		}
	}

	print $cgi->end_html();

}
###############################################################################

sub getPage{
	my ($cgi,$j) = @_;
	my $pagename = $cgi->param('page');
	my $page     = URI::Escape::uri_escape( $j->set($pagename)->utf8 );

	my $result   = 
	    XMLRPC::Lite->proxy($URI)->call(
	    	'wiki.getPage', $page
	    )->result;

	print $cgi->h2( "getPage" );
	print $cgi->h4( jcode($pagename)->sjis );
	print $cgi->hr;
	print $cgi->pre(
		$j->set($result)->sjis
	);
	print $cgi->hr;

	return $result;
}


sub getPageHTML{
	my ($cgi,$j) = @_;
	my $pagename = $cgi->param('page');
	my $page     = URI::Escape::uri_escape( $j->set($pagename)->utf8 );

	my $result   = 
	    XMLRPC::Lite->proxy($URI)->call(
	    	'wiki.getPageHTML', $page
	    )->result;

	print $cgi->h2( "getPageHTML" );
	print $cgi->h4( jcode($pagename)->sjis );
	print $cgi->hr;
	print $j->set($result)->sjis;
	print $cgi->hr;

	return $result;
}

sub getAllPages{
	my ($cgi,$j) = @_;
	my $path = $cgi->param('farm');

	print $cgi->h2( "getAllPages" );
	print $cgi->hr;

	my $result = 
	 XMLRPC::Lite->proxy($URI)->call(
	 	'wiki.getAllPages', $path
	 )->result;

	my $self_url = $cgi->url . '?method=getPageHTML&amp;page=';
	my $farm     = $cgi->param('farm') ?  $cgi->param('farm') . ':' : '';

	print "<ul>"; # CGI.pmȤʸ
	for my $page ( @$result){
		my $name = $j->set(URI::Escape::uri_unescape($page),'utf8')->sjis;
		print qq|<li><a href="$self_url$farm$name">|;
		print $name;
		print qq|</a></li>|;
	}
	print "</ul>";

	print $cgi->hr;

	return $result;
}

sub listLinks{
	my ($cgi,$j) = @_;
	my $pagename = $cgi->param('page');
	my $page     = URI::Escape::uri_escape( $j->set($pagename)->utf8 );

	my $result = 
	 XMLRPC::Lite->proxy($URI)->call(
	 	'wiki.listLinks', $page
	 )->result;

	print $cgi->h2( "listLinks" );
	print $cgi->h4( jcode($pagename)->sjis );
	print $cgi->hr;

	print "<ul>"; # CGI.pmȤʸ
	for my $item (@$result){
		my $page = $j->set(URI::Escape::uri_unescape($item->{page}),'utf8')->sjis;
		my $href = $item->{href};
		print qq|<li><a href="$href">|;
		print $page;
		print qq|</a></li>|;
	}
	print "</ul>";

	print $cgi->hr;

	return $result;
}

sub addPage{
	$_[0]->param('add',1);
	putPage(@_);
}

sub putPage{
	my ($cgi,$j,$hashref) = @_;
	my $pagename = $cgi->param('page');
	my $sage     = $cgi->param('sage');
	my $add      = $cgi->param('add');
	my $id       = $cgi->param('id');
	my $pass     = $cgi->param('pass');
	my $content  = '';

	my $page     = URI::Escape::uri_escape( $j->set($pagename)->utf8 );

	if($add){
		$content = 
		    XMLRPC::Lite->proxy($URI)->call('wiki.getPage', $page)->result;
		$content .= "\n";
		print $cgi->h2( "addPage" );
	}
	else{
		print $cgi->h2( "putPage" );
	}
	$content .= $cgi->param('content');

	my $encoded  = $j->set($content)->utf8;
	my $attr     = { minoredit => $sage, id => $id, pass => $pass};

	my $result = 
	 XMLRPC::Lite->proxy($URI)->call(
	 	'wiki.putPage', $page, $encoded, $attr
	 )->result;

	if($result){
		my $html = 
		 XMLRPC::Lite->proxy($URI)->call(
		 	'wiki.getPageHTML', $page)->result;
		print $cgi->h4( jcode($pagename)->sjis );
		print $cgi->hr;
		print $j->set($content)->sjis;
		#print $j->set($html)->sjis;
		print $cgi->hr;
	}
	else{
		print $cgi->h4( jcode($pagename)->sjis );
		print $cgi->hr;
		print $cgi->p('Not successful');
		print $cgi->hr;
	}
	return $result;
}

sub getRecentChanges{
	my ($cgi,$j) = @_;
	my $day  = $cgi->param('day') || 1;
	my $path = $cgi->param('farm');
	my $date = HTTP::Date::time2isoz(time - 3600*24*$day);

	$date =~ s/[-Z]//g;
	$date =~ tr/ /T/;

	my $result = 
	 XMLRPC::Lite->proxy($URI)->call(
	 	'wiki.getRecentChanges', $date, $path
	 )->result;

	print $cgi->h2( "getRecentChanges" );
	print $cgi->h4("since " . $date);

	my $self_url = $cgi->url . '?method=getPageHTML&amp;page=';
	my $farm     = $cgi->param('farm') ?  $cgi->param('farm') . ':' : '';

	print "<ul>"; # CGI.pmȤʸ
	for my $page (@$result){
		my $name = $j->set(URI::Escape::uri_unescape($page->{name}),'utf8')->sjis;
		my $modtime = $page->{lastModified};
		print qq|<li><a href="$self_url$farm$name">|;
		print $name;
		print qq|</a> $modtime</li>|;
	}
	print "</ul>";

	print $cgi->hr;
}
