package WikiAPI::WikiMod;
###############################################################################
# Wiki XML-RPC API for FreeStyle Wiki
#      Wiki module wrapper (FSWiki 3.5.3 base)
# v.1.00
#
###############################################################################
use strict;
use lib './lib';
use CGI::Carp qw(fatalsToBrowser);
use CGI2;
use Wiki;
use Util;
use HTML::Template;
use Wiki::HTMLParser;
use WikiAPI::LinkParser;
use HTTP::Date;
###############################################################################

sub new{
	my $class = shift;
	my $self  = {};
	my %opt   = @_;
	bless($self,$class);
	$self->set_code($opt{code});
	$self->{wiki}  = $self->setup();
	$self->{jcode} = undef; # cache Jcode instance
	return $self;
}

#
# setup Wiki instance and return it.
#

sub setup{
	my $class = shift;
	my $cgi   = CGI2->new();
	my $wiki  = Wiki->new($cgi,'setup.dat');

	$wiki->config('session_dir',$wiki->config('log_dir'));

	my $config = &Util::load_config_hash($wiki,$wiki->config('config_file'));
	foreach my $key (keys(%$config)){
		$wiki->config($key,$config->{$key});
	}

	my $cache_config = &Util::load_config_hash($wiki,'cache.dat');
	$wiki->config('use_cache'   ,$cache_config->{use_cache});
	$wiki->config('no_cache'    ,$cache_config->{no_cache});
	$wiki->config('remove_cache',$cache_config->{remove_cache});

	my @plugins = split(/\n/,&Util::load_config_text($wiki,$wiki->config('plugin_file')));
	my $plugin_error = '';
	foreach(sort(@plugins)){
		$plugin_error .= $wiki->install_plugin($_);
	}

	$wiki->do_hook("initialize");

	return $wiki;
}


#
# set moji code
#

sub set_code{
	my $self = shift;
	my $code = shift;

	return $self->{code} = 'utf8' if(!defined $code);
	return $self->{code} = 'utf8'
	    if($code ne 'utf8' and $code ne 'euc' and $code ne 'sjis');

	my $old = $self->{code};
	$self->{code} = $code;
	return $old;
}

#
# clean up Wiki instance. but this can be use FSWiki 3.5.3 and after.
#

sub DESTROY{
	$_[0]->exit(); # before 3.5.2, this must be commented out
}

###############################################################################
# Wiki XML-RPC API for FSWiki Implementaion
###############################################################################

#
# get pagename and return its source text.
#

sub get_page{
	my $self = shift;
	my $page = $self->decode_page( shift );
	return $self->{wiki}->get_page($page);
}

#
# get pagename and return its text rendered with html.
#

sub get_page_html{
	my $self = shift;
	my $page = $self->decode_page( shift );
	my $wiki = $self->{wiki};
	$wiki->get_CGI->param(page => $page);
	return $self->encode_content( $wiki->call_handler() );
}

#
# get epoch time and return a listref of hashref which contains
# information of pages changed since that time.
# this method can take wikifarm name.
#

sub recent_changes {
	my ($self,$time,$path) = @_;
	my $wiki   = $self->{wiki};
	my $cgi    = $wiki->get_CGI;
	my $max    = 200; # --test
	my @pages;

	$self->set_path($path);

	for my $page ($wiki->get_page_list({-sort=>'last_modified',-permit=>'show'})){
		my $modtime = $wiki->get_last_modified2($page);
		my ($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime($modtime);
		$year += 1900;
		$mon  += 1;

		last unless($max--);
		next if($modtime < $time);

		my $date = $self->time2utc($modtime);
		$page    = $self->encode_page($page);

		push @pages,{
			name         => $page,
			lastModified => $date,
			author       => '',
			version      => 1,
		};
	}

	return \@pages;
}

#
# return a listref of all pages.
# this method can take wikifarm name.
#

sub get_all_pages{
	my ($self,$path) = @_;
	my $wiki = $self->{wiki};

	$self->set_path($path);

	my @list = $wiki->get_page_list({-sort=>'last_modified',-permit=>'show'});
	my @pages;

	for my $page (@list){
		push @pages, $self->encode_page($page);
	}

	return \@pages;
}

#
# get page name and return a hashref which contains
# information of the page.
#

sub get_page_info{
	my $self = shift;
	my $page = $self->decode_page( shift );
	my $wiki = $self->{wiki};
	my $path;

	my $modtime = $wiki->get_last_modified2($page);
	my $date    = $self->time2utc($modtime);

	$page = $self->encode_page($page);

	return {
		name         => $page,
		lastModified => $date,
		author       => '',
		version      => 1,
	};
}

#
# get page name and return a listref of hashref which contains
# link information of the page.
# this info can be gotten by LinkParser.
#

sub list_links{
	my $self  = shift;
	my $page  = $self->decode_page( shift );
	my $wiki  = $self->{wiki};
	my ($path)  = $wiki->get_CGI->url =~ /^(.+)\/.+$/;

	$wiki->config('script_name', $path . '/' . $wiki->config('script_name'));

	my $source = $self->{wiki}->get_page($page);
	my $parser = WikiAPI::LinkParser->new(
	                 $wiki,$self->get_jcode,$self->{code}
	             );

	$parser->parse($source);
	my $links = $parser->get_links();

	return $links;
}

#
# get page name, its content and attributes. then write wiki data.
# if writing is successful, return true. otherwise false.
#

sub put_page{
	my $self    = shift;
	my $page    = $self->decode_page( shift );
	my $content = $self->decode_content( shift );
	my $attr    = shift;
	my $wiki    = $self->{wiki};
	my $cgi     = $wiki->get_CGI;
	my $path;

	my $modtime = $wiki->get_last_modified($page);

	my $wikiname = ref($wiki) . '::redirect';
	no strict 'refs';
	local *{$wikiname} = sub { return; };
	$wikiname = ref($wiki) . '::get_login_info';
	local *{$wikiname} = \&get_login_info;

	$cgi->param('page',$page);
	$cgi->param('content',$content);
	$cgi->param('save',1);
	$cgi->param('lastmodified',$modtime);
	$cgi->param('sage',$attr->{minoredit});
	$cgi->param('id',,$attr->{id});
	$cgi->param('pass',$attr->{pass});

	my $result = $wiki->call_handler('EDIT');

	if( $result =~  m{<div class="error">} ){
		#print STDERR $result;
		return 0;
	}
	else{
		return 1;
	}
}

###############################################################################

#
# Wiki::get_login_info() dummy subroutine
#

sub get_login_info{
	my $wiki = shift;
	my $cgi  = $wiki->get_CGI;
	my $id   = $cgi->param('id');
	my $pass = $cgi->param('pass');
	my $path;

	my $hashref = Util::load_config_hash($wiki,'user.dat');
	my $check   = Util::md5($pass,$id);

	my ($user_pass,$type) = split/\t/,$hashref->{$id};

	if( $user_pass eq $check ){
		return {id=>$id,type=>$type,path=>$path};
	}
	else{
		return undef;
	}
}

###############################################################################
# Utility Methods
###############################################################################

sub split_path{
	my $self = shift;
	my $page = shift;
	my $wiki = $self->{wiki};
	my $path;

	$self->reset_path() if($self->{set_path});

	return $page unless(defined $page and $page =~ /:/);

	($path,$page) = split(/:/,$page);
	$self->set_path($path);
	return $page;
}

sub reset_path{
	my $self = shift;
	my $wiki = $self->{wiki};

	for my $name ( keys %{$self->{old_path}} ){
		$wiki->config($name, $self->{old_path}->{$name});
	}

	$self->{set_path} = 0;
}

sub set_path{
	my $self = shift;
	my $path = shift || return;
	my $wiki = $self->{wiki};

	for my $name (qw/data_dir config_dir backup_dir log_dir script_name/){
		$self->{old_path}->{$name} = $wiki->config($name);
		$wiki->config($name, $wiki->config($name).'/'.$path);
	}

	$self->{set_path} = 1;
}

sub get_jcode{
	return (defined $_[0]->{jcode})
	 ? $_[0]->{jcode} : ($_[0]->{jcode} = new Jcode);
}

sub encode_page{
	my $self  = shift;
	my $page  = shift;
	my $ocode = $self->{code};
	my $j     = $self->get_jcode();

	if(!defined $page){ return; }

	return Util::url_encode( $j->set($page,'euc')->$ocode() );
}

sub decode_page{
	my $self  = shift;
	my $page  = shift;
	my $icode = $self->{code};
	my $j     = $self->get_jcode();

	$page = (defined $page) ? URI::Escape::uri_unescape($page) : return '';
	$page =  $j->set($page,$icode)->euc;
	return $self->split_path($page);
}

sub decode_content{
	my $self    = shift;
	my $content = shift;
	my $icode   = $self->{code};
	my $j       = $self->get_jcode();
	return $j->set($content,$icode)->euc;
}

sub encode_content{
	my $self    = shift;
	my $content = shift;
	my $ocode   = $self->{code};
	my $j       = $self->get_jcode();

	return $j->set($content,'euc')->$ocode();
}

sub time2utc{
	shift if(ref $_[0]);
	my $time = shift;
	my $date = HTTP::Date::time2isoz($time);
	$date =~ s/[-Z]//g;
	$date =~ tr/ /T/;
	return $date;
}

sub exit{
	$_[0]->{wiki}->_process_before_exit();
}

###############################################################################
1;
