#!/usr/bin/perl
###############################################################################
$| = 1;
use strict;
use lib qw(./lib);
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use Jcode;
use WikiAPI::WikiMod; # Wrapper for FSWiki
use URI::Escape;

use vars qw(%Dispatch $CODE);
%Dispatch = (
	getrecentchanges => \&getRecentChanges,
	recentchanges    => \&getRecentChanges,
	getpage          => \&getPage,
	getpagehtml      => \&getPageHTML,
	putpage          => \&putPage,
	addpage          => \&addPage,
	add              => \&addPage,
	put              => \&putPage,
	getallpages      => \&getAllPages,
	listlinks        => \&listLinks,
);

if(exists $ENV{MOD_PERL}){ Jcode::load_module("Jcode::Unicode"); }
###############################################################################

$CODE = 'sjis';

###############################################################################
main();
sub main{
	my $cgi    = new CGI;
	my $j      = new Jcode;
	my $method = $cgi->param('method') || $cgi->param('m');

	print $cgi->header( -type  =>'text/html; charset=Shift_JIS', );
	print $cgi->start_html('FSWiki Gate');

	if( defined( my $sub = $Dispatch{lc($method)}) ){
		$sub->($cgi,$j);
	}
	else{
		if(!defined $method or $method eq ''){
			print $cgi->h1("Method must be specified.");
		}
		else{
			print $cgi->h1("$method is not defined.");
		}
	}

	print $cgi->end_html();
}
###############################################################################

sub getPage{
	my ($cgi,$j) = @_;
	my $page     = $cgi->param('page');
	my $pagename = URI::Escape::uri_escape( $j->set($page)->euc );
	my $wiki     = new WikiAPI::WikiMod 'code' => 'euc';
	header($cgi,$j,'getPage',$page);
	print $cgi->pre( $j->set($wiki->get_page($pagename))->$CODE() );
	print $cgi->hr;
}

sub getPageHTML{
	my ($cgi,$j) = @_;
	my $page     = $cgi->param('page');
	my $pagename = URI::Escape::uri_escape( $j->set($page)->euc );
	my $wiki     = new WikiAPI::WikiMod 'code' => 'euc';
	header($cgi,$j,'getPageHTML',$page);
	print $j->set( $wiki->get_page_html($pagename) )->$CODE();
	print $cgi->hr;
}

sub getRecentChanges{
	my ($cgi,$j) = @_;
	my $day    = $cgi->param('day') || 1;
	my $path   = $cgi->param('farm');
	my $time   = time - 3600 * 24 * $day;
	my $date   = localtime($time);

	my $wiki   = new WikiAPI::WikiMod 'code' => 'euc';
	my $result = $wiki->recent_changes($time,$path);

	header($cgi,$j,'getRecentChanges', "since $date");

	my $url  = $cgi->url;
	my $farm = $cgi->param('farm') ?  $cgi->param('farm') . ':' : '';

	print "<ul>"; # CGI.pmȤʸ
	for my $page (@$result){
		my $name = $j->set(URI::Escape::uri_unescape($page->{name}))->sjis;
		my $modtime = $page->{lastModified};
		print qq|<li><a href="$url?method=getPageHTML&amp;page=$farm$name">|;
		print qq|$name</a> |;
		print qq|<a href="$url?method=getPage&amp;page=$farm$name">|;
		print qq|[source]</a> $modtime</li>|;
	}
	print "</ul>";
	print $cgi->hr;
}


sub getAllPages{
	my ($cgi,$j) = @_;
	my $path     = $cgi->param('farm');
	my $wiki     = new WikiAPI::WikiMod 'code' => 'euc';
	my $result   = $wiki->get_all_pages($path);
	my $url      = $cgi->url;
	my $farm     = $cgi->param('farm') ?  $cgi->param('farm') . ':' : '';

	header($cgi,$j,'getAllPages');

	print "<ul>"; # CGI.pmȤʸ
	for my $page ( @$result){
		my $name = $j->set(URI::Escape::uri_unescape($page))->sjis;
		print qq|<li><a href="$url?method=getPageHTML&amp;page=$farm$name">|;
		print $name;
		print qq|</a> |;
		print qq|<a href="$url?method=getPage&amp;page=$farm$name">|;
		print qq|[source]</a></li>|;
	}
	print "</ul>";
	print $cgi->hr;
}

sub listLinks{
	my ($cgi,$j) = @_;
	my $page     = $cgi->param('page');
	my $pagename = URI::Escape::uri_escape( $j->set($page)->utf8 );
	my $wiki     = new WikiAPI::WikiMod 'code' => 'euc';
	my $result   = $wiki->list_links($page);

	header($cgi,$j,'listLinks',$page);

	print "<ul>"; # CGI.pmȤʸ
	for my $item (@$result){
		my $page = $j->set(URI::Escape::uri_unescape($item->{page}),'euc')->sjis;
		my $href = $item->{href};
		print qq|<li><a href="$href">|;
		print $page;
		print qq|</a></li>|;
	}
	print "</ul>";
	print $cgi->hr;
}

sub addPage{
	$_[0]->param('add',1);
	putPage(@_);
}

sub putPage{
	my ($cgi,$j) = @_;
	my $page     = $cgi->param('page') || $cgi->param('n');
	my $sage     = $cgi->param('sage') || 0;
	my $add      = $cgi->param('add');
	my $id       = $cgi->param('id')   || $cgi->param('i');
	my $pass     = $cgi->param('pass') || $cgi->param('p');
	my $wiki     = new WikiAPI::WikiMod 'code' => 'euc';
	my $content  = '';
	my $method   = 'putPage';

	my $pagename = URI::Escape::uri_escape( $j->set($page)->euc );

	if($add){
		$content = $wiki->get_page($pagename);
		$content  = $j->set($content)->euc if(defined $content and $content eq '');
		$method  = 'addPage';
	}
	$content .= "\n" . $j->set(
	  ( $cgi->param('content') || $cgi->param('c') ) )->euc;

	my $encoded  = $j->set($content)->euc;
	my $attr     = { minoredit => $sage, id => $id, pass => $pass};

	$wiki->put_page($pagename,$encoded,$attr);

	header($cgi,$j,$method,$page);
	print $j->set( $wiki->get_page_html($pagename) )->$CODE();
	print $cgi->hr;
}

sub header{
	my ($cgi,$j,$method,$subtitle) = @_;
	print $cgi->h2( "$method" );
	print $cgi->h4( $j->set($subtitle)->$CODE() ) if(defined $subtitle);
	print $cgi->hr;
}

